@@ -48,9 +48,6 @@ func TestGetObjectCore(t *testing.T) {
 		t.Skip("skipping functional tests for the short runs")
 	}
 
-	// Seed random based on current time.
-	rand.Seed(time.Now().Unix())
-
 	// Instantiate new minio core client object.
 	c, err := NewCore(
 		os.Getenv(serverEndpoint),
@@ -248,9 +245,6 @@ func TestGetObjectContentEncoding(t *testing.T) {
 		t.Skip("skipping functional tests for the short runs")
 	}
 
-	// Seed random based on current time.
-	rand.Seed(time.Now().Unix())
-
 	// Instantiate new minio core client object.
 	c, err := NewCore(
 		os.Getenv(serverEndpoint),
@@ -324,9 +318,6 @@ func TestGetBucketPolicy(t *testing.T) {
 		t.Skip("skipping functional tests for short runs")
 	}
 
-	// Seed random based on current time.
-	rand.Seed(time.Now().Unix())
-
 	// Instantiate new minio client object.
 	c, err := NewCore(
 		os.Getenv(serverEndpoint),
@@ -390,9 +381,6 @@ func TestCoreCopyObject(t *testing.T) {
 		t.Skip("skipping functional tests for short runs")
 	}
 
-	// Seed random based on current time.
-	rand.Seed(time.Now().Unix())
-
 	// Instantiate new minio client object.
 	c, err := NewCore(
 		os.Getenv(serverEndpoint),
@@ -516,9 +504,6 @@ func TestCoreCopyObjectPart(t *testing.T) {
 		t.Skip("skipping functional tests for short runs")
 	}
 
-	// Seed random based on current time.
-	rand.Seed(time.Now().Unix())
-
 	// Instantiate new minio client object.
 	c, err := NewCore(
 		os.Getenv(serverEndpoint),
@@ -672,9 +657,6 @@ func TestCorePutObject(t *testing.T) {
 		t.Skip("skipping functional tests for short runs")
 	}
 
-	// Seed random based on current time.
-	rand.Seed(time.Now().Unix())
-
 	// Instantiate new minio client object.
 	c, err := NewCore(
 		os.Getenv(serverEndpoint),
