@@ -1,4 +1,5 @@
-// +build ignore
+//go:build example
+// +build example
 
 /*
  * MinIO Go Library for Amazon S3 Compatible Cloud Storage
@@ -76,7 +77,7 @@ func main() {
 	}
 
 	// Create slice of sources.
-	srcs := []minio.SourceInfo{src1, src2, src3}
+	srcs := []minio.CopySrcOptions{src1, src2, src3}
 
 	// Prepare destination encryption key
 	encKey, _ := encrypt.NewSSEC([]byte{8, 9, 0})
